<?php

namespace Drupal\tfl\Controller;

use Drupal\Core\Controller\ControllerBase;


/**
 * Class AccountChecker.
 *
 * @package Drupal\tfl\Controller\AccountChecker
 */
class AccountChecker extends ControllerBase {

  /**
   * Salt $secret_key.
   *
   * @var $secret_key
   */
  protected $secret_key = 'Key#&PdkrtlG$789fS$tlG$77d';

  /**
   * Salt $secret_iv.
   *
   * @var $secret_iv
   */
  protected $secret_iv = 'Iv#&PdkrtlG$789fS$tlG$77d';

  /**
   * The password hashing service.
   *
   * @var \Drupal::service('password')
   */
  protected $password_hasher;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManagerInterface
   */
  protected $entityManager;

  /*
   * Constructor to create password object.
   */
  public function __construct() {
    $this->password_hasher = \Drupal::service( 'password' );
    $this->entityManager = \Drupal::entityTypeManager();
  }

  /**
   * {@inheritdoc}
   */
  public function authenticate($username, $password) {
    $user = FALSE;
    $account = '';
    if (!empty( $username ) && strlen( $password ) > 0) {
      //$account_search = user_load_by_name($username);
      $account_search = $this->entityManager->getStorage( 'user' )
        ->loadByProperties( ['name' => $username] );
      if ($account = reset( $account_search )) {
        if ($this->password_hasher->check( $password, $account->getPassword() )) {
          $user = $account;
        }
      }
    }

    return $user;
  }


  /*
   * Get hashed passowrd from user field data table.
   * 
   * @params  Username
   */
  public function getHashedPassword($username) {
    $query = \Drupal::database()->select( 'users_field_data', 'ufd' );
    $query->fields( 'ufd', ['pass'] );
    $query->condition( 'ufd.name', $username );
    $result = $query->execute()->fetchField();

    return $result;
  }

  /*
   * Get OTP from user field data table.
   * 
   * @params  userId
   */
  public function getOtpSessionId($uid) {
    $query = \Drupal::database()->select( 'users_otp_data', 'uod' );
    $query->fields( 'uod', ['otp_session_id'] );
    $query->condition( 'uod.uid', $uid );
    $result = $query->execute()->fetchField();

    return $result;
  }

  /**
   * Verify otp session id from user field data table.
   *
   * @params
   * $uid  user id
   * $otp_input  user otp
   */
  public function isOtpVerified($uid, $otp) {
    $config = \Drupal::config( 'tfl.settings' );
    // Authorized 2factor.in API Key.
    $api = $config->get( 'apikey' );
    // Enable status to login with OTP.
    $status = $config->get( 'status' );
    // OTP Message type is SMS OR VOICE.
    $otpType = $config->get( 'type' );

    $encodedSessionId = $this->getOtpSessionId( $uid );
    $sessionId = $this->encryptOrDecrypt( 'decrypt', $encodedSessionId );
    $url = 'https://2factor.in/API/V1/' . $api . '/' . $otpType . '/VERIFY/' . $sessionId . '/' . $otp;
    $result = self::get2FactorApiData( $url );

    return $result;
  }

  /*
   * Get OTP from user field data table.
   * 
   * @params  
   * $uid   integer
   * $otp_session_id  string
   * @return  boolean
   */
  public function isOtpSessionIdVerified($uid, $otp_session_id) {
    $status = FALSE;
    $query = \Drupal::database()->select( 'users_otp_data', 'uod' );
    $query->fields( 'uod', ['uid'] );
    $query->condition( 'uod.uid', $uid );
    $query->condition( 'uod.otp_session_id', $otp_session_id );
    $result = $query->execute()->fetchField();
    if ((int) $result > 0) {
      $status = TRUE;
    }

    return $status;
  }

  /*
   * Get API data using cURL.
   * 
   * @params  
   * $url   api url
   */
  public static function get2FactorApiData($url) {
    $ch = curl_init();
    curl_setopt( $ch, CURLOPT_URL, $url );
    curl_setopt( $ch, CURLOPT_RETURNTRANSFER, TRUE );
    $response = curl_exec( $ch );
    curl_close( $ch );
    $result = json_decode( $response );

    return $result;
  }


  /*
   * Clear user otp data table.
   * 
   * @params  userId
   */
  public function clearUserOtpData($uid) {
    $query = \Drupal::database()->delete( 'users_otp_data' );
    $query->condition( 'uid', $uid )->execute();
  }

  /**
   * simple method to encrypt or decrypt a plain text string
   * initialization vector(IV) has to be the same when encrypting and decrypting
   *
   * @param string $action : can be 'encrypt' or 'decrypt'
   * @param string $string : string to encrypt or decrypt
   *
   * @return string
   */
  public function encryptOrDecrypt($action, $string) {
    $output = FALSE;
    $encrypt_method = "AES-256-CBC";
    // hash
    $key = hash( 'sha256', $this->secret_key );
    // iv - encrypt method AES-256-CBC expects 16 bytes - else you will get a warning
    $iv = substr( hash( 'sha256', $this->secret_iv ), 0, 16 );
    if ($action == 'encrypt') {
      $output = openssl_encrypt( $string, $encrypt_method, $key, 0, $iv );
      $output = base64_encode( $output );
    }
    else {
      if ($action == 'decrypt') {
        $output = openssl_decrypt( base64_decode( $string ), $encrypt_method, $key, 0, $iv );
      }
    }

    return $output;
  }

}      
