<?php

namespace Drupal\tfl\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\HttpFoundation\JsonResponse;


/**
 * Class Login2Factor.
 *
 * @package Drupal\tfl\Controller\Login2Factor
 */
class Login2Factor extends ControllerBase {


  /**
   * {@inheritdoc}
   */
  public function sendOtp($username) {
    $response = self::getResponse( $username );

    return $response;
  }

  /**
   * {@inheritdoc}
   */
  public function getResponse($username) {
    $config = \Drupal::config( 'tfl.settings' );
    // Authorized 2factor.in API Key.
    $api = $config->get( 'apikey' );
    // Enable status to login with OTP.
    $status = $config->get( 'status' );
    // OTP Message type is SMS OR VOICE.
    $otpType = $config->get( 'type' );

    $account = user_load_by_name( $username );
    //kint($account);exit;
    $mobile = !is_null( $account->field_mobile_number->value ) ? $account->field_mobile_number->value : '';
    $accountChecker = new AccountChecker();
    //set POST variables
    $url = 'https://2factor.in/API/V1/' . $api . '/' . $otpType . '/' . $mobile . '/AUTOGEN';
    $response = AccountChecker::get2FactorApiData( $url );
    $result = isset( $response->Status ) ? $response->Status : 'Failed';
    if ($result == 'Success' && !empty( $mobile )) {
      $otp_session_id = $accountChecker->encryptOrDecrypt( 'encrypt', $response->Details );
      self::insertUserOtpData( $account->uid->value, $otp_session_id );
    }

    return new JsonResponse( $response );
  }


  /**
   * Insert session id for OTP.
   *
   * @params
   * $uid  user id
   * $otp_session_id  otp session id
   */
  public function insertUserOtpData($uid, $otp_session_id) {
    $result = \Drupal::database()->merge( 'users_otp_data' )
      ->key( ['uid' => $uid] )
      ->insertFields( [
        'uid' => $uid,
        'otp_session_id' => $otp_session_id,
      ] )
      ->updateFields( [
        'otp_session_id' => $otp_session_id,
      ] )->execute();

    return $result;
  }


}
